@@ -21,7 +21,10 @@
 #include "defs.h"
 #include "frame.h"
 #include "regcache.h"
+#include "reggroups.h"
 #include "value.h"
+#include "target.h"
+#include "gdb_string.h"
 
 #include <tcl.h>
 #include "gdbtk.h"
@@ -32,29 +35,36 @@
 
    It is an array of (NUM_REGS+NUM_PSEUDO_REGS)*MAX_REGISTER_RAW_SIZE bytes. */
 
-static char *old_regs = NULL;
-
-static int get_pc_register (ClientData, Tcl_Interp *, int, Tcl_Obj * CONST[]);
-static int gdb_register_info (ClientData, Tcl_Interp *, int, Tcl_Obj * CONST[]);
+static int gdb_register_info (ClientData, Tcl_Interp *, int, Tcl_Obj **);
 static void get_register (int, void *);
 static void get_register_name (int, void *);
 static void get_register_size (int regnum, void *arg);
-static int map_arg_registers (int, Tcl_Obj * CONST[],
+static int map_arg_registers (Tcl_Interp *, int, Tcl_Obj **,
 			      void (*)(int, void *), void *);
 static void register_changed_p (int, void *);
 static void setup_architecture_data (void);
+static int gdb_regformat (ClientData, Tcl_Interp *, int, Tcl_Obj **);
+static int gdb_reggroup (ClientData, Tcl_Interp *, int, Tcl_Obj **);
+static int gdb_reggrouplist (ClientData, Tcl_Interp *, int, Tcl_Obj **);
+
+static void get_register_types (int regnum, void *arg);
+
+static char *old_regs = NULL;
+static int *regformat = (int *)NULL;
+static struct type **regtype = (struct type **)NULL;
 
 int
 Gdbtk_Register_Init (Tcl_Interp *interp)
 {
   Tcl_CreateObjCommand (interp, "gdb_reginfo", gdbtk_call_wrapper,
                         gdb_register_info, NULL);
-  Tcl_CreateObjCommand (interp, "gdb_pc_reg", gdbtk_call_wrapper, get_pc_register,
-			NULL);
 
   /* Register/initialize any architecture specific data */
   setup_architecture_data ();
+
   register_gdbarch_swap (&old_regs, sizeof (old_regs), NULL);
+  register_gdbarch_swap (&regformat, sizeof (regformat), NULL);
+  register_gdbarch_swap (&regtype, sizeof (regtype), NULL);
   register_gdbarch_swap (NULL, 0, setup_architecture_data);
 
   return TCL_OK;
@@ -102,31 +112,33 @@ Gdbtk_Register_Init (Tcl_Interp *interp)
  * value
  *    Returns a list of register values.
  *
- *    usage: gdb_reginfo value format [regnum0, ..., regnumN]
- *       format: The format string for printing the values, "N", "x", "d", etc
+ *    usage: gdb_reginfo value [regnum0, ..., regnumN]
  */
 static int
 gdb_register_info (ClientData clientData, Tcl_Interp *interp, int objc,
-                   Tcl_Obj *CONST objv[])
+                   Tcl_Obj **objv)
 {
   int index;
   void *argp;
   void (*func)(int, void *);
-  static char *commands[] = {"changed", "name", "size", "value", NULL};
-  enum commands_enum { REGINFO_CHANGED, REGINFO_NAME, REGINFO_SIZE, REGINFO_VALUE };
+  static const char *commands[] = {"changed", "name", "size", "value", "type", 
+			     "format", "group", "grouplist", NULL};
+  enum commands_enum { REGINFO_CHANGED, REGINFO_NAME, REGINFO_SIZE, REGINFO_VALUE, 
+		       REGINFO_TYPE, REGINFO_FORMAT, REGINFO_GROUP, REGINFO_GROUPLIST };
 
   if (objc < 2)
     {
-      Tcl_WrongNumArgs (interp, 1, objv, "name|size|value [regnum1 ... regnumN]");
+      Tcl_WrongNumArgs (interp, 1, objv, "name|size|value|type|format|groups [regnum1 ... regnumN]");
       return TCL_ERROR;
     }
 
   if (Tcl_GetIndexFromObj (interp, objv[1], commands, "options", 0,
-			   &index) != TCL_OK)
+  			   &index) != TCL_OK)
     {
+      result_ptr->flags |= GDBTK_IN_TCL_RESULT;
       return TCL_ERROR;
     }
-
+  
   /* Skip the option */
   objc -= 2;
   objv += 2;
@@ -162,16 +174,28 @@ gdb_register_info (ClientData clientData, Tcl_Interp *interp, int objc,
 
     case REGINFO_VALUE:
       func = get_register;
-      argp = (void *) (int) *(Tcl_GetStringFromObj (objv[0], NULL));
-      objc--;
-      objv++;
+      argp = NULL;
       break;
 
+    case REGINFO_TYPE:
+      func = get_register_types;
+      argp = NULL;
+      break;
+
+    case REGINFO_FORMAT:
+      return gdb_regformat (clientData, interp, objc, objv);
+
+    case REGINFO_GROUP:
+      return gdb_reggroup (clientData, interp, objc, objv);
+
+    case REGINFO_GROUPLIST:
+      return gdb_reggrouplist (clientData, interp, objc, objv);
+
     default:
       return TCL_ERROR;
     }
 
-  return map_arg_registers (objc, objv, func, argp);
+  return map_arg_registers (interp, objc, objv, func, argp);
 }
 
 static void
@@ -181,43 +205,94 @@ get_register_size (int regnum, void *arg)
 			    Tcl_NewIntObj (REGISTER_RAW_SIZE (regnum)));
 }
 
-/* This implements the tcl command get_pc_reg
- * It returns the value of the PC register
- *
- * Tcl Arguments:
- *    None
- * Tcl Result:
- *    The value of the pc register.
- */
-static int
-get_pc_register (ClientData clientData, Tcl_Interp *interp,
-		 int objc, Tcl_Obj *CONST objv[])
-{
-  char *buff;
-  xasprintf (&buff, "0x%s", paddr_nz (read_register (PC_REGNUM)));
-  Tcl_SetStringObj (result_ptr->obj_ptr, buff, -1);
-  free(buff);
-  return TCL_OK;
-}
+/* returns a list of valid types for a register */
+/* Normally this will be only one type, except for SIMD and other */
+/* special registers. */
 
 static void
-get_register (int regnum, void *fp)
-{
+get_register_types (int regnum, void *arg)
+{ 
   struct type *reg_vtype;
-  char raw_buffer[MAX_REGISTER_RAW_SIZE];
-  char virtual_buffer[MAX_REGISTER_VIRTUAL_SIZE];
-  int format = (int) fp;
-  int optim;
+  int i,n;
 
-  if (format == 'N')
-    format = 0;
+  reg_vtype = register_type (current_gdbarch, regnum);
+  
+  if (TYPE_CODE (reg_vtype) == TYPE_CODE_UNION)
+    {
+      n = TYPE_NFIELDS (reg_vtype);
+      /* limit to 16 types */
+      if (n > 16) 
+	n = 16;
+      
+      for (i = 0; i < n; i++)
+	{
+	  Tcl_Obj *ar[3], *list;
+	  char *buff;
+	  xasprintf (&buff, "%lx", (long)TYPE_FIELD_TYPE (reg_vtype, i));
+	  ar[0] = Tcl_NewStringObj (TYPE_FIELD_NAME (reg_vtype, i), -1);
+	  ar[1] = Tcl_NewStringObj (buff, -1);
+	  if (TYPE_CODE (TYPE_FIELD_TYPE (reg_vtype, i)) == TYPE_CODE_FLT)
+	    ar[2] = Tcl_NewStringObj ("float", -1);
+	  else
+	    ar[2] = Tcl_NewStringObj ("int", -1);	    
+	  list = Tcl_NewListObj (3, ar);
+	  Tcl_ListObjAppendElement (gdbtk_interp, result_ptr->obj_ptr, list);
+	  xfree (buff);
+	}
+    }
+  else
+    {
+      Tcl_Obj *ar[3], *list;
+      char *buff;
+      xasprintf (&buff, "%lx", (long)reg_vtype);
+      ar[0] = Tcl_NewStringObj (TYPE_NAME(reg_vtype), -1);
+      ar[1] = Tcl_NewStringObj (buff, -1);
+      if (TYPE_CODE (reg_vtype) == TYPE_CODE_FLT)
+	ar[2] = Tcl_NewStringObj ("float", -1);
+      else
+	ar[2] = Tcl_NewStringObj ("int", -1);	    
+      list = Tcl_NewListObj (3, ar);
+      xfree (buff);
+      Tcl_ListObjAppendElement (gdbtk_interp, result_ptr->obj_ptr, list);
+    }
+}
+
+
+static void
+get_register (int regnum, void *arg)
+{
+  int realnum;
+  CORE_ADDR addr;
+  enum lval_type lval;
+  struct type *reg_vtype;
+  char raw_buffer[MAX_REGISTER_SIZE];
+  char virtual_buffer[MAX_REGISTER_SIZE];
+  int optim, format;
+  struct cleanup *old_chain = NULL;
+  struct ui_file *stb;
+  long dummy;
+  char *res;
+ 
+  format = regformat[regnum];
+  if (format == 0)
+    format = 'x';
+  
+  reg_vtype = regtype[regnum];
+  if (reg_vtype == NULL)
+    reg_vtype = register_type (current_gdbarch, regnum);
+
+  if (!target_has_registers)
+    {
+      if (result_ptr->flags & GDBTK_MAKES_LIST)
+	Tcl_ListObjAppendElement (NULL, result_ptr->obj_ptr, Tcl_NewStringObj ("", -1));
+      else
+	Tcl_SetStringObj (result_ptr->obj_ptr, "", -1);
+      return;
+    }
+
+  frame_register (get_selected_frame (), regnum, &optim, &lval, 
+		  &addr, &realnum, raw_buffer);
 
-  /* read_relative_register_raw_bytes returns a virtual frame pointer
-     (FRAME_FP (selected_frame)) if regnum == FP_REGNUM instead
-     of the real contents of the register. To get around this,
-     use get_saved_register instead. */
-  get_saved_register (raw_buffer, &optim, (CORE_ADDR *) NULL, selected_frame,
-		      regnum, (enum lval_type *) NULL);
   if (optim)
     {
       Tcl_ListObjAppendElement (NULL, result_ptr->obj_ptr,
@@ -226,18 +301,20 @@ get_register (int regnum, void *fp)
     }
 
   /* Convert raw data to virtual format if necessary.  */
-
-  reg_vtype = REGISTER_VIRTUAL_TYPE (regnum);
-  if (REGISTER_CONVERTIBLE (regnum))
+  if (DEPRECATED_REGISTER_CONVERTIBLE (regnum))
     {
-      REGISTER_CONVERT_TO_VIRTUAL (regnum, reg_vtype,
+      DEPRECATED_REGISTER_CONVERT_TO_VIRTUAL (regnum, reg_vtype,
       				   raw_buffer, virtual_buffer);
     }
   else
     memcpy (virtual_buffer, raw_buffer, REGISTER_VIRTUAL_SIZE (regnum));
 
+  stb = mem_fileopen ();
+  old_chain = make_cleanup_ui_file_delete (stb);
+
   if (format == 'r')
     {
+      /* shouldn't happen. raw format is deprecated */
       int j;
       char *ptr, buf[1024];
 
@@ -245,24 +322,36 @@ get_register (int regnum, void *fp)
       ptr = buf + 2;
       for (j = 0; j < REGISTER_RAW_SIZE (regnum); j++)
 	{
-	  register int idx = TARGET_BYTE_ORDER == BFD_ENDIAN_BIG ? j
+	  int idx = TARGET_BYTE_ORDER == BFD_ENDIAN_BIG ? j
 	    : REGISTER_RAW_SIZE (regnum) - 1 - j;
 	  sprintf (ptr, "%02x", (unsigned char) raw_buffer[idx]);
 	  ptr += 2;
 	}
-      fputs_filtered (buf, gdb_stdout);
+      fputs_unfiltered (buf, stb);
     }
   else
-    if ((TYPE_CODE (reg_vtype) == TYPE_CODE_UNION)
-        && (strcmp (FIELD_NAME (TYPE_FIELD (reg_vtype, 0)), REGISTER_NAME (regnum)) == 0))
-      {
-        val_print (FIELD_TYPE (TYPE_FIELD (reg_vtype, 0)), virtual_buffer, 0, 0,
-	           gdb_stdout, format, 1, 0, Val_pretty_default);
-      }
-    else
-      val_print (REGISTER_VIRTUAL_TYPE (regnum), virtual_buffer, 0, 0,
-	         gdb_stdout, format, 1, 0, Val_pretty_default);
+    {
+      if ((TYPE_CODE (reg_vtype) == TYPE_CODE_UNION)
+	  && (strcmp (FIELD_NAME (TYPE_FIELD (reg_vtype, 0)), 
+		      REGISTER_NAME (regnum)) == 0))
+	{
+	  val_print (FIELD_TYPE (TYPE_FIELD (reg_vtype, 0)), virtual_buffer, 0, 0,
+		     stb, format, 1, 0, Val_pretty_default);
+	}
+      else
+	val_print (reg_vtype, virtual_buffer, 0, 0,
+		   stb, format, 1, 0, Val_pretty_default);
+    }
+  
+  res = ui_file_xstrdup (stb, &dummy);
 
+  if (result_ptr->flags & GDBTK_MAKES_LIST)
+    Tcl_ListObjAppendElement (NULL, result_ptr->obj_ptr, Tcl_NewStringObj (res, -1));
+  else
+    Tcl_SetStringObj (result_ptr->obj_ptr, res, -1);
+
+  xfree (res);
+  do_cleanups (old_chain);
 }
 
 static void
@@ -292,9 +381,8 @@ get_register_name (int regnum, void *argp)
 /* This is a sort of mapcar function for operations on registers */
 
 static int
-map_arg_registers (int objc, Tcl_Obj *CONST objv[],
-		   void (*func) (int regnum, void *argp),
-		   void *argp)
+map_arg_registers (Tcl_Interp *interp, int objc, Tcl_Obj **objv,
+		   void (*func) (int regnum, void *argp), void *argp)
 {
   int regnum, numregs;
 
@@ -306,25 +394,26 @@ map_arg_registers (int objc, Tcl_Obj *CONST objv[],
 
   numregs = NUM_REGS + NUM_PSEUDO_REGS;
 
-  if (objc == 0 || objc > 1)
-    result_ptr->flags |= GDBTK_MAKES_LIST;
-
   if (objc == 0)		/* No args, just do all the regs */
     {
-      for (regnum = 0;
-	   regnum < numregs;
-	   regnum++)
+      result_ptr->flags |= GDBTK_MAKES_LIST;
+      for (regnum = 0; regnum < numregs; regnum++)
 	{
 	  if (REGISTER_NAME (regnum) == NULL
 	      || *(REGISTER_NAME (regnum)) == '\0')
 	    continue;
-	  
 	  func (regnum, argp);
-	}
-      
+	}      
       return TCL_OK;
     }
 
+  if (objc == 1)
+    if (Tcl_ListObjGetElements (interp, *objv, &objc, &objv ) != TCL_OK)
+      return TCL_ERROR;
+
+  if (objc > 1)
+    result_ptr->flags |= GDBTK_MAKES_LIST;
+
   /* Else, list of register #s, just do listed regs */
   for (; objc > 0; objc--, objv++)
     {
@@ -334,8 +423,7 @@ map_arg_registers (int objc, Tcl_Obj *CONST objv[],
 	  return TCL_ERROR;
 	}
 
-      if (regnum >= 0
-	  && regnum < numregs
+      if (regnum >= 0  && regnum < numregs
 	  && REGISTER_NAME (regnum) != NULL
 	  && *REGISTER_NAME (regnum) != '\000')
 	func (regnum, argp);
@@ -345,25 +433,25 @@ map_arg_registers (int objc, Tcl_Obj *CONST objv[],
 	  return TCL_ERROR;
 	}
     }
-
   return TCL_OK;
 }
 
 static void
 register_changed_p (int regnum, void *argp)
 {
-  char raw_buffer[MAX_REGISTER_RAW_SIZE];
+  char raw_buffer[MAX_REGISTER_SIZE];
 
-  if (read_relative_register_raw_bytes (regnum, raw_buffer))
+  if (deprecated_selected_frame == NULL
+      || !frame_register_read (deprecated_selected_frame, regnum, raw_buffer))
     return;
 
-  if (memcmp (&old_regs[regnum * MAX_REGISTER_RAW_SIZE], raw_buffer,
+  if (memcmp (&old_regs[regnum * MAX_REGISTER_SIZE], raw_buffer,
 	      REGISTER_RAW_SIZE (regnum)) == 0)
     return;
 
   /* Found a changed register.  Save new value and return its number. */
 
-  memcpy (&old_regs[regnum * MAX_REGISTER_RAW_SIZE], raw_buffer,
+  memcpy (&old_regs[regnum * MAX_REGISTER_SIZE], raw_buffer,
 	  REGISTER_RAW_SIZE (regnum));
 
   Tcl_ListObjAppendElement (NULL, result_ptr->obj_ptr, Tcl_NewIntObj (regnum));
@@ -372,10 +460,118 @@ register_changed_p (int regnum, void *argp)
 static void
 setup_architecture_data ()
 {
-  if (old_regs != NULL)
-    xfree (old_regs);
+  xfree (old_regs);
+  xfree (regformat);
+  xfree (regtype);
 
-  old_regs = xmalloc ((NUM_REGS + NUM_PSEUDO_REGS) * MAX_REGISTER_RAW_SIZE + 1);
-  memset (old_regs, 0, sizeof  (old_regs));
+  old_regs = xcalloc (1, (NUM_REGS + NUM_PSEUDO_REGS) * MAX_REGISTER_SIZE + 1);
+  regformat = (int *)xcalloc ((NUM_REGS + NUM_PSEUDO_REGS) , sizeof(int));
+  regtype = (struct type **)xcalloc ((NUM_REGS + NUM_PSEUDO_REGS), sizeof(struct type **));
+}
+
+/* gdb_regformat sets the format for a register */
+/* This is necessary to allow "gdb_reginfo value" to return a list */
+/* of registers and values. */
+/* Usage: gdb_reginfo format regno typeaddr format */
+
+static int
+gdb_regformat (ClientData clientData, Tcl_Interp *interp,
+	       int objc, Tcl_Obj **objv)
+{
+  int fm, regno;
+  struct type *type;
+
+  if (objc != 3)
+    {
+      Tcl_WrongNumArgs (interp, 0, objv, "gdb_reginfo regno type format");
+      return TCL_ERROR;
+    }
+
+  if (Tcl_GetIntFromObj (interp, objv[0], &regno) != TCL_OK)
+    return TCL_ERROR;
+
+  type = (struct type *)strtol (Tcl_GetStringFromObj (objv[1], NULL), NULL, 16);  
+  fm = (int)*(Tcl_GetStringFromObj (objv[2], NULL));
+
+  if (regno >= NUM_REGS + NUM_PSEUDO_REGS)
+    {
+      gdbtk_set_result (interp, "Register number %d too large", regno);
+      return TCL_ERROR;
+    }
+  
+  regformat[regno] = fm;
+  regtype[regno] = type;
+
+  return TCL_OK;
+}
+
+
+/* gdb_reggrouplist returns the names of the register groups */
+/* for the current architecture. */
+/* Usage: gdb_reginfo groups */
+
+static int
+gdb_reggrouplist (ClientData clientData, Tcl_Interp *interp,
+		  int objc, Tcl_Obj **objv)
+{
+  struct reggroup *const *groups;
+  int i = 0;
+
+  if (objc != 0)
+    {
+      Tcl_WrongNumArgs (interp, 0, objv, "gdb_reginfo grouplist");
+      return TCL_ERROR;
+    }
+
+  groups = reggroups (current_gdbarch);
+
+  while (groups[i] != NULL) {
+    if (reggroup_type (groups[i]) == USER_REGGROUP)
+      Tcl_ListObjAppendElement (NULL, result_ptr->obj_ptr, Tcl_NewStringObj (reggroup_name(groups[i]), -1));
+    i++;
+  }
+  return TCL_OK;
+}
+
+
+/* gdb_reggroup returns the names of the registers in a group. */
+/* Usage: gdb_reginfo group groupname */
+
+static int
+gdb_reggroup (ClientData clientData, Tcl_Interp *interp,
+	      int objc, Tcl_Obj **objv)
+{
+  struct reggroup *const *group;
+  char *groupname;
+  int regnum;
+
+  if (objc != 1)
+    {
+      Tcl_WrongNumArgs (interp, 0, objv, "gdb_reginfo group groupname");
+      return TCL_ERROR;
+    }
+  
+  groupname = Tcl_GetStringFromObj (objv[0], NULL);
+  if (groupname == NULL)
+    {
+      gdbtk_set_result (interp, "could not read groupname");
+      return TCL_ERROR;
+    }
+
+  for (group = reggroups (current_gdbarch); *group != NULL; group++)
+    {
+      if (strcmp (groupname, reggroup_name (*group)) == 0)
+	break;
+    }
+
+  if (*group == NULL)
+    return TCL_ERROR;
+
+  for (regnum = 0; regnum < NUM_REGS + NUM_PSEUDO_REGS; regnum++)
+    {
+      if (gdbarch_register_reggroup_p (current_gdbarch, regnum, *group))
+	Tcl_ListObjAppendElement (NULL, result_ptr->obj_ptr, Tcl_NewIntObj (regnum));
+    }
+  return TCL_OK;
 }
 
